package com.eitc.patient.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eitc.common.core.domain.AjaxResult;
import com.eitc.patient.domain.AppointmentProject;
import com.eitc.patient.mapper.AppointmentProjectMapper;
import com.eitc.patient.service.IAppointmentProjectService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * AppointmentProjectServiceImpl
 *
 * @author mtx
 * @date: 2024/05/17 15:39
 */
@Service
public class AppointmentProjectServiceImpl extends ServiceImpl<AppointmentProjectMapper, AppointmentProject> implements IAppointmentProjectService {

    /**
     * 删除预约项目
     *
     * @param ids 删除参数
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public AjaxResult delete(String ids) {
        if(StringUtils.isEmpty(ids)){
            return AjaxResult.error("请选择要删除的记录");
        }
        List<String> idList = Arrays.asList(ids.split(","));
        deleteAll(idList);
        return AjaxResult.success();
    }

    /**
     * 递归删除
     * @param idList ids
     * @author mtx
     * @date: 2024/6/21 10:12
     */
    @Transactional(rollbackFor = {Exception.class})
    public void deleteAll(List<String> idList){
        for (String id : idList) {
            List<AppointmentProject> list = this.list(Wrappers.<AppointmentProject>lambdaQuery().eq(AppointmentProject::getParentId, id));
            if(CollectionUtils.isNotEmpty(list)){
                deleteAll(list.stream().map(AppointmentProject::getId).collect(Collectors.toList()));
            }
            this.removeById(id);
        }
    }
}
