package com.eitc.patient.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eitc.common.core.domain.entity.SysUser;
import com.eitc.common.utils.bean.BeanUtils;
import com.eitc.patient.domain.PatientAppointment;
import com.eitc.patient.service.IPatientAppointmentService;
import com.eitc.patient.service.IDoctorService;
import com.eitc.patient.vo.DoctorVO;
import com.eitc.system.mapper.SysUserMapper;
import com.eitc.system.utils.UserCacheUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * DoctorServiceImpl
 *
 * @author mtx
 * @date: 2024/06/13 14:58
 */
@Service
public class DoctorServiceImpl implements IDoctorService {

    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private IPatientAppointmentService patientAppointmentService;

    /**
     * 医生列表
     *
     * @return 返回结果
     * @author mtx
     * @date: 2024/6/13 15:01
     */
    @Override
    public List<DoctorVO> list() {
        Date date = new Date();
        List<SysUser> sysUsers = sysUserMapper.selectUserList(new SysUser());
        if (CollectionUtils.isNotEmpty(sysUsers)) {
            List<DoctorVO> list = new ArrayList<>();
            for (SysUser sysUser : sysUsers) {
                DoctorVO vo = new DoctorVO();
                BeanUtils.copyProperties(sysUser, vo);
                long count = patientAppointmentService.count(Wrappers.<PatientAppointment>lambdaQuery()
                        .lt(PatientAppointment::getAppointmentStartTime, date)
                        .gt(PatientAppointment::getAppointmentEndTime, date)
                        .eq(PatientAppointment::getAttendingDoctorId, sysUser.getUserId()));

                vo.setIsFree(count > 0 ? 1 : 0);
                vo.setDeptName(UserCacheUtil.getSysDeptById(sysUser.getDeptId()).getDeptName());
                list.add(vo);
            }
            return list;
        }

        return new ArrayList<>();
    }
}
