package com.eitc.patient.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eitc.common.utils.bean.BeanUtils;
import com.eitc.patient.domain.MedicalHistory;
import com.eitc.patient.domain.MedicalRecordDetails;
import com.eitc.patient.enums.MedicalHistoryDiseaseSourceEnum;
import com.eitc.patient.enums.MedicalHistoryDiseaseTypeEnum;
import com.eitc.patient.mapper.MedicalRecordDetailsMapper;
import com.eitc.patient.service.IMedicalHistoryService;
import com.eitc.patient.service.IMedicalRecordDetailsService;
import com.eitc.patient.vo.MedicalRecordDetailsVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * MedicalRecordDetailsServiceImpl
 *
 * @author mtx
 * @date: 2024/05/17 15:39
 */
@Service
public class MedicalRecordDetailsServiceImpl extends ServiceImpl<MedicalRecordDetailsMapper, MedicalRecordDetails> implements IMedicalRecordDetailsService {

    @Resource
    private IMedicalHistoryService medicalHistoryService;

    @Override
    public List<MedicalRecordDetailsVO> findMedicalRecordDetailsVOByMedicalRecordId(String medicalRecordId) {
        List<MedicalRecordDetails> list = this.list(Wrappers.<MedicalRecordDetails>lambdaQuery()
                .eq(MedicalRecordDetails::getMedicalRecordId, medicalRecordId)
                .orderByAsc(MedicalRecordDetails::getOrderNum));
        if (CollectionUtils.isNotEmpty(list)) {
            // 封装病例详情数据
            return list.stream().map(item -> {
                MedicalRecordDetailsVO detailsVO = new MedicalRecordDetailsVO();
                BeanUtils.copyProperties(item, detailsVO);
                if (MedicalHistoryDiseaseTypeEnum.ALLERGY_HISTORY.getInfo().equals(detailsVO.getTemplateName())) {
                    findMedicalHistory(medicalRecordId, MedicalHistoryDiseaseTypeEnum.ALLERGY_HISTORY.getCode(), detailsVO);
                }
                if (MedicalHistoryDiseaseTypeEnum.PAST_HISTORY.getInfo().equals(detailsVO.getTemplateName())) {
                    findMedicalHistory(medicalRecordId, MedicalHistoryDiseaseTypeEnum.PAST_HISTORY.getCode(), detailsVO);
                }
                if (MedicalHistoryDiseaseTypeEnum.FAMILY_HISTORY.getInfo().equals(detailsVO.getTemplateName())) {
                    findMedicalHistory(medicalRecordId, MedicalHistoryDiseaseTypeEnum.FAMILY_HISTORY.getCode(), detailsVO);
                }
                return detailsVO;
            }).collect(Collectors.toList());
        } else {
            return Collections.emptyList();
        }
    }

    private void findMedicalHistory(String medicalRecordId, Integer diseaseType, MedicalRecordDetailsVO detailsVO) {
        List<MedicalHistory> histories = medicalHistoryService.list(Wrappers.<MedicalHistory>lambdaQuery()
                .eq(MedicalHistory::getDiseaseType, diseaseType)
                .eq(MedicalHistory::getMedicalRecordId, medicalRecordId));

        detailsVO.setPatientMedicalRecord(histories.stream()
                .filter(li -> MedicalHistoryDiseaseSourceEnum.PATIENT.getCode().equals(li.getDiseaseSource()))
                .map(MedicalHistory::getDiseaseContent).collect(Collectors.joining(",")));
        detailsVO.setDoctorMedicalRecord(histories.stream()
                .filter(li -> MedicalHistoryDiseaseSourceEnum.DOCTOR.getCode().equals(li.getDiseaseSource()))
                .map(MedicalHistory::getDiseaseContent).collect(Collectors.joining(",")));

        String patientMedicalRecord = detailsVO.getPatientMedicalRecord();
        String doctorMedicalRecord = detailsVO.getDoctorMedicalRecord();
        if (StringUtils.isNotBlank(patientMedicalRecord) && StringUtils.isNotBlank(doctorMedicalRecord)) {
            detailsVO.setContentInfo(patientMedicalRecord + "，" + doctorMedicalRecord);
        } else if (StringUtils.isNotBlank(patientMedicalRecord)) {
            detailsVO.setContentInfo(patientMedicalRecord);
        } else if (StringUtils.isNotBlank(doctorMedicalRecord)) {
            detailsVO.setContentInfo(doctorMedicalRecord);
        }
    }
}
