package com.eitc.patient.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eitc.common.utils.PatientBaseUtil;
import com.eitc.common.utils.bean.BeanUtils;
import com.eitc.patient.domain.MedicalHistory;
import com.eitc.patient.domain.Patient;
import com.eitc.patient.enums.MedicalHistoryDiseaseSourceEnum;
import com.eitc.patient.mapper.PatientMapper;
import com.eitc.patient.service.IMedicalHistoryService;
import com.eitc.patient.service.IPatientService;
import com.eitc.patient.vo.PatientVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * PatientServiceImpl
 *
 * @author mtx
 * @date: 2024/05/17 15:40
 */
@Service
public class PatientServiceImpl extends ServiceImpl<PatientMapper, Patient> implements IPatientService {

    @Resource
    private IMedicalHistoryService medicalHistoryService;


    /**
     * 添加患者
     *
     * @param vo 参数
     * @author mtx
     * @date: 2024/5/22 9:12
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public Patient addOrEdit(PatientVO vo) {
        Patient patient = new Patient();
        BeanUtils.copyProperties(vo, patient);
        if (StringUtils.isEmpty(patient.getId())) {
            patient.setMedicalRecordNum(PatientBaseUtil.findMedicalRecordNum());
//        } else {
//            patient.setMedicalRecordNum(null);
        }
        this.saveOrUpdate(patient);

        medicalHistoryService.saveMedicalHistoryList(
                vo.getAllergyHistory(),
                vo.getPastHistory(),
                vo.getFamilyHistory(),
                null,
                vo.getId(),
                MedicalHistoryDiseaseSourceEnum.PATIENT.getCode());
        return patient;
    }

    /**
     * 删除患者
     *
     * @param ids ids
     * @return 状态
     * @author mtx
     * @date: 2024/5/22 9:40
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public boolean delete(List<String> ids) {
        this.removeBatchByIds(ids);
        return medicalHistoryService.remove(Wrappers.<MedicalHistory>lambdaQuery().in(MedicalHistory::getPatientId, ids));
    }
}
