package com.eitc.patient.controller;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eitc.common.annotation.Log;
import com.eitc.common.core.controller.BaseController;
import com.eitc.common.core.domain.AjaxResult;
import com.eitc.common.enums.BusinessType;
import com.eitc.common.utils.PatientBaseUtil;
import com.eitc.common.utils.PdfUtil;
import com.eitc.common.utils.bean.BeanUtils;
import com.eitc.patient.domain.DiagnosisType;
import com.eitc.patient.domain.MedicalRecord;
import com.eitc.patient.domain.MedicalRecordDetails;
import com.eitc.patient.domain.Patient;
import com.eitc.patient.dto.PatientTeethInfoDto;
import com.eitc.patient.enums.MedicalRecordStatusEnum;
import com.eitc.patient.service.IDiagnosisTypeService;
import com.eitc.patient.service.IMedicalRecordDetailsService;
import com.eitc.patient.service.IMedicalRecordService;
import com.eitc.patient.service.IPatientService;
import com.eitc.patient.utils.PatientUtil;
import com.eitc.patient.vo.MedicalRecordDetailsVO;
import com.eitc.patient.vo.MedicalRecordParamVO;
import com.eitc.patient.vo.MedicalRecordTemplateVO;
import com.eitc.patient.vo.MedicalRecordVO;
import com.eitc.system.utils.UserCacheUtil;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.nio.charset.StandardCharsets;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * MedicalRecordController
 *
 * @author mtx
 * @date: 2024/05/17 15:52
 */
@Api(tags = "病例")
@RestController
@RequestMapping("/patient/medical-record")
public class MedicalRecordController extends BaseController {

    @Resource
    private IMedicalRecordService medicalRecordService;
    @Resource
    private IMedicalRecordDetailsService medicalRecordDetailsService;
    @Resource
    private IPatientService patientService;
    @Resource
    private IDiagnosisTypeService diagnosisTypeService;

    @Resource
    private FreeMarkerConfigurer freeMarkerConfigurer;

    /**
     * 获取患者详细信息
     */
    @PostMapping(value = "/html2Pdf")
    @ApiOperation("打印病历")
    public void html2Pdf(@RequestBody MedicalRecordTemplateVO vo, HttpServletResponse response) throws IOException, TemplateException {
        Map<String, List<MedicalRecordDetailsVO>> listMap = vo.getListMap();
        for (String s : listMap.keySet()) {
            List<MedicalRecordDetailsVO> vos = listMap.get(s);
            if (CollectionUtils.isEmpty(vos)) {
                continue;
            }
            List<MedicalRecordDetailsVO> collect = vos.stream().filter(MedicalRecordDetailsVO::isFlagCheck).collect(Collectors.toList());
            listMap.put(s, collect);
        }

        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("companyName", UserCacheUtil.getClinicName() + "口腔科病历");
        paramMap.put("patient", vo);

        Writer out = new StringWriter();
        //获取模板地址
        Template template = freeMarkerConfigurer.getConfiguration().getTemplate("medical-record.html");
        template.process(paramMap, out);
        out.flush();
        out.close();
        String templateContent = out.toString();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/pdf");
        String fileName = System.currentTimeMillis() + ".pdf";
        response.setHeader("Content-Disposition", "filename=" + new String(fileName.getBytes(), StandardCharsets.UTF_8));

        byte[] resources = PdfUtil.html2Pdf(templateContent);
        OutputStream outputStream = response.getOutputStream();
        outputStream.write(resources);
        outputStream.close();
    }

    @GetMapping("/findMedicalInfo")
    @ApiOperation("打印病历详情")
    public AjaxResult findMedicalInfo(String medicalRecordId) throws ParseException {
        MedicalRecordTemplateVO mrt = new MedicalRecordTemplateVO();
        MedicalRecord medicalRecord = medicalRecordService.getById(medicalRecordId);
        if (Objects.nonNull(medicalRecord)) {
            String patientId = medicalRecord.getPatientId();
            Patient patient = patientService.getById(patientId);
            if (Objects.nonNull(patient)) {
                BeanUtils.copyProperties(patient, mrt);
                mrt.setAge(PatientBaseUtil.findAge(patient.getIdentificationCard()));
            }
            List<MedicalRecordDetailsVO> list = medicalRecordDetailsService.findMedicalRecordDetailsVOByMedicalRecordId(medicalRecordId);
            Map<String, List<MedicalRecordDetailsVO>> collect = list.stream().collect(Collectors.groupingBy(MedicalRecordDetailsVO::getTemplateName));
            Map<String, List<MedicalRecordDetailsVO>> linkedMap = new LinkedHashMap<>();
            for (String s : PatientUtil.diagnosisTypeTemplateKeyList()) {
                if ("主诉".equals(s) || "现病史".equals(s) || "既往史".equals(s) || "家族史".equals(s) || "过敏史".equals(s)) {
                    List<MedicalRecordDetailsVO> vos = collect.get(s);
                    if (CollectionUtils.isNotEmpty(vos)) {
                        linkedMap.put(s, Collections.singletonList(vos.get(0)));
                    } else {
                        linkedMap.put(s, new ArrayList<>());
                    }
                } else {
                    linkedMap.put(s, collect.get(s));
                }
            }
            mrt.setListMap(linkedMap);
        }
        return success(mrt);
    }

    /**
     * 查询病历
     */
    @GetMapping("/list")
    @ApiOperation("查询病历")
    public AjaxResult list(MedicalRecord medicalRecord) {
        List<MedicalRecord> list = medicalRecordService.list(Wrappers.<MedicalRecord>lambdaQuery()
                .eq(MedicalRecord::getPatientId, medicalRecord.getPatientId())
                .orderByDesc(MedicalRecord::getCreateDate));

        if (Objects.isNull(list)) {
            return success(new ArrayList<MedicalRecordVO>());
        }
        List<MedicalRecordParamVO> voList = new ArrayList<>();
        for (MedicalRecord record : list) {
            voList.add(medicalRecordService.getMedicalRecordVO(record));
        }
        return success(voList);
    }

    /**
     * 获取病历详细信息
     */
    @GetMapping(value = "/detail")
    @ApiOperation("获取病历详细信息")
    public AjaxResult detail(String id) {
        MedicalRecord medicalRecord = medicalRecordService.getById(id);
        return success(medicalRecordService.getMedicalRecordVO(medicalRecord));
    }

    /**
     * 新增病历
     */
    @Log(title = "新增病历", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增病历")
    public AjaxResult add(@Valid @RequestBody MedicalRecordParamVO vo) {
        if (CollectionUtils.isEmpty(vo.getDataList())) {
            return error("无数据");
        }
        medicalRecordService.addOrEdit(vo);
        return success();
    }

    /**
     * 修改病历
     */
    @Log(title = "修改病历", businessType = BusinessType.UPDATE)
    @PutMapping
    @ApiOperation("修改病历")
    public AjaxResult edit(@Valid @RequestBody MedicalRecordParamVO vo) {
        if (StringUtils.isEmpty(vo.getId())) {
            return error();
        }
        if (CollectionUtils.isEmpty(vo.getDataList())) {
            return error("无数据");
        }
        medicalRecordService.addOrEdit(vo);
        return success();
    }

    /**
     * 删除病历
     */
    @Log(title = "删除病历", businessType = BusinessType.DELETE)
    @DeleteMapping("/remove")
    @ApiOperation("删除病历")
    public AjaxResult remove(String ids) {
        List<String> idList = Arrays.asList(ids.split(","));
        return toAjax(medicalRecordService.removeBatchByIds(idList));
    }

    /**
     * 根据患者id查询牙齿治疗状态
     *
     * @param patientId 患者id
     * @return 返回结果
     * @author mtx
     * @date: 2024/6/18 10:11
     */
    @GetMapping(value = "/teethListByPatientId")
    @ApiOperation("根据患者id查询牙齿治疗状态")
    public AjaxResult teethListByPatientId(String patientId) {
        List<MedicalRecord> list = medicalRecordService.list(Wrappers.<MedicalRecord>lambdaQuery().eq(MedicalRecord::getPatientId, patientId));
        if (CollectionUtils.isEmpty(list)) {
            return success(new ArrayList<>());
        }
        List<PatientTeethInfoDto> dtoList = new ArrayList<>();
        for (MedicalRecord medicalRecord : list) {
            PatientTeethInfoDto dto = new PatientTeethInfoDto();
            Integer status = medicalRecord.getStatus();
            List<MedicalRecordDetails> recordDetails = medicalRecordDetailsService.list(Wrappers.<MedicalRecordDetails>lambdaQuery().eq(MedicalRecordDetails::getMedicalRecordId, medicalRecord.getId()));
            if (CollectionUtils.isEmpty(recordDetails)) {
                return success(new ArrayList<>());
            }
            Optional<MedicalRecordDetails> first = recordDetails.stream().findFirst();
            if (first.isPresent()) {
                MedicalRecordDetails medicalRecordDetails = first.get();
                dto.setPatientId(patientId);
                dto.setToothPositionStatus(status);
                dto.setToothPosition(medicalRecordDetails.getToothPosition());
                dto.setMedicalRecordId(medicalRecord.getId());
                dto.setMedicalRecordDetailsId(medicalRecordDetails.getId());
                DiagnosisType diagnosisType = diagnosisTypeService.getById(medicalRecordDetails.getDiagnosisTypeId());
                if (Objects.nonNull(diagnosisType)) {
                    dto.setDiagnosisTypeName(diagnosisType.getDiagnosisName());
                }
                dto.setCreateDate(medicalRecord.getCreateDate());
                dto.setStatus(MedicalRecordStatusEnum.WAIT_COMPLETE.getCode().equals(status) ?
                        MedicalRecordStatusEnum.WAIT_COMPLETE.getInfo() : MedicalRecordStatusEnum.ALREADY_COMPLETE.getInfo());
                dtoList.add(dto);
            }
        }

        return success(dtoList);
    }

}
